//
//  LLSpilloDataController.h
//  Spillo
//
//  Created by Damien DeVille on 4/14/14.
//  Copyright (c) 2014 Damien DeVille. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>

@class LLSpilloSearch, LLSpilloPinboardPost;

@interface LLSpilloDataController : NSObject

/*!
	\brief
	A managed object context of concurrency type `NSMainQueueConcurrencyType` tied to the data store.
 */
@property (readonly, strong, nonatomic) NSManagedObjectContext *mainManagedObjectContext;

#pragma mark - Authentication

/*!
	\brief
	YES if the user is authenticated, NO otherwise. Observable.
 */
@property (readonly, getter = isAuthenticated, assign, nonatomic) BOOL authenticated;
extern NSString * const LLSpilloDataControllerAuthenticatedKey;

/*!
	\brief
	The username of the currently authenticated user.
 */
@property (readonly, copy, nonatomic) NSString *username;
extern NSString * const LLSpilloDataControllerUsernameKey;

/*!
	\brief
	Start authentication for a given username and API token
	
	\param username
	The user's username. Required.
	
	\param token
	The user's API token. Required.
 
	\param completionHandler
	A block that will be invoked when the authentication completes or fails.
 */
- (void)authenticateWithUsername:(NSString *)username token:(NSString *)token completionHandler:(void (^)(BOOL authenticated, NSError *error))completionHandler;

/*!
	\brief
	Start authentication for a given username and password.
	
	\param username
	The user's username. Required.
	
	\param password
	The user's password. Required.
	
	\param completionHandler
	A block that will be invoked when the authentication completes or fails.
 */
- (void)authenticateWithUsername:(NSString *)username password:(NSString *)password completionHandler:(void (^)(BOOL authenticated, NSError *error))completionHandler;

/*!
	\brief
	Log out the current user.
	
	\param completionHandler
	A block that will be invoked when the logout completes or fails.
 */
- (void)logout:(void (^)(BOOL success, NSError *error))completionHandler;

#pragma mark - Refresh

/*!
	\brief
	YES if the controller is currently refreshing, NO otherwise. Observable. Changes on the main thread.
 */
@property (readonly, getter = isRefreshing, assign, atomic) BOOL refreshing;
extern NSString * const LLSpilloDataControllerRefreshingKey;

/*!
	\brief
	The current refresh status. Observable. Changes on the main thread.
 */
@property (readonly, copy, nonatomic) NSString *refreshStatus;
extern NSString * const LLSpilloDataControllerRefreshStatusKey;

/*!
	\brief
	The interval between each automatic refresh.
 */
@property (assign, nonatomic) NSTimeInterval refreshInterval;
extern NSString * const LLSpilloDataControllerRefreshIntervalKey;

/*!
	\brief
	Whether dead links should be checked when refreshing.
 */
@property (assign, nonatomic) BOOL checkDeadLinks;
extern NSString * const LLSpilloDataControllerCheckDeadLinksKey;

/*!
	\brief
	Start refreshing the user's data.
	
	\param completionHandler
	A block that will be invoked when the refresh completes or fails.
 */
- (id)refresh:(void (^)(BOOL refreshed, NSError *error))completionHandler;

#pragma mark - Search

/*!
	\brief
	Start retrieving a list of posts for a given search.
	
	\param search
	The search to be performed.
	
	\param completionHandler
	A block that will be invoked when the search completes or fails.
	
	\return
	A token that can be used to cancel the search by calling `-cancelSearch:`.
 */
- (id)retrievePostsForSearch:(LLSpilloSearch *)search completionHandler:(void (^)(NSArray *posts, NSError *error))completionHandler;

#pragma mark - Cancellation

/*!
	\brief
	Cancel an operation.
	
	\param token
	A token that was returned by previously calling a work method.
 */
- (void)cancelOperation:(id)token;

@end
